#include "Mhisto.h"
#include <limits>
#include <QMessageBox>
#include <QFileDialog>
#include "DataProcessorFactory.hpp"

static QColor colors[] = {
	QColor(QString("lime")),
	QColor(QString("blue")),
	QColor(QString("blueviolet")),
	QColor(QString("coral")),
	QColor(QString("cyan")),
	QColor(QString("crimson")),
	QColor(QString("gold")),
	QColor(QString("teal")),
	QColor(QString("seagreen")),
	QColor(QString("red")),
	QColor(QString("orchid")),
	QColor(QString("orange")),
	QColor(QString("magenta")),
	QColor(QString("aqua")),
	QColor(QString("chocolate")),
	QColor(QString("pink"))
};

Mhisto::Mhisto(QWidget *parent)
: QwtPlot(parent), accumulation(false)
{
	QwtPlotGrid *grid = new QwtPlotGrid;
	grid->setPen(QPen(Qt::gray, 1, Qt::DashLine));
	grid->attach(this);

	setCanvasBackground(QColor(Qt::white));

	for (int i=0; i<CHANNELS; i++) {
		curv[i] = new QwtPlotCurve(QString("curve") + QString::number(i+1));
		curv[i]->attach(this);
		curv[i]->setAxes(yLeft, xBottom);
		curv[i]->setStyle(QwtPlotCurve::Steps);
	}
	for (int i=0; i<CHANNELS; i++) {
		curv[i]->setPen(QPen(colors[i]));
		curv[i]->setBrush(QBrush(colors[i]));
	}
	panner = new QwtPlotPanner(canvas());
	panner->setMouseButton(Qt::MidButton);

	setAxisTitle(QwtPlot::xBottom, "Time, [ns]");
	setAxisTitle(QwtPlot::yLeft, "Counts");

	zoomer = new QwtPlotZoomer(canvas());
	zoomer->setMousePattern(
		QwtEventPattern::MouseSelect2,
		Qt::RightButton,
		Qt::ControlModifier
	);
	zoomer->setMousePattern(
		QwtEventPattern::MouseSelect3,
		Qt::RightButton
	);
	replot();
}

int Mhisto::init_data_processor()
{
	int ret;
	sc_tdc_format f;
	ret = sc_tdc_get_format(&f);
	if (ret < 0) {
		QMessageBox::critical(this, "Dll Error", "Could not get data format descriptor");
		return ret;
	}
	try {
		data_processor_.reset(DataProcessorFactory::create(f, data_store));
	} catch (DataProcessorFactoryException &e) {
		return e.code();
	}
	return 0;
}

void
Mhisto::newBuffer
(unsigned int *buf, int total_events, int binning, long long histosize_ns)
{
	int maxY = std::numeric_limits <unsigned> ::min();
	long long histosize = ns_to_bin(1. *histosize_ns);
	if (!(accumulation && data_store.histosize == histosize)) {
		data_store.reset();
	}
	data_store.total_events += total_events;
	data_store.histosize = histosize;

	if (data_processor_.get() == 0) {
		QMessageBox::critical(this, "Error", "No DataProcessor initialized");
		return;
	}

	data_processor_->resetX();
	maxY = data_processor_->process(buf, binning);
	emit takeRates(data_store.rates);

	for (unsigned i=0; i<CHANNELS; ++i) {
		data_store.field[i][0] = 0.;
		data_store.field[i][histosize] = 0.;
		int size = data_store.field[i].size();
		double *x = new double[size];
		double *y = new double[size];
		QList <double> keys = data_store.field[i].keys();
		QList <double> values = data_store.field[i].values();
		for (int j=0; j<size; ++j) {
			x[j] = bin_to_ns(keys.at(j));
			y[j] = values.at(j);
		}

		curv[i]->setSamples(x, y, size);
		delete [] x;
		delete [] y;
	}
	int currentIndex = zoomer->zoomRectIndex();
	QStack <QRectF> stack;
	QRectF rect;
	if (data_processor_->maxX() > data_processor_->minX()) {
		rect.setX(0);
		rect.setY(0);
		rect.setWidth(histosize_ns);
		rect.setHeight(maxY);
	} else {
		rect.setX(0);
		rect.setY(0);
		rect.setWidth(histosize_ns);
		rect.setHeight(100);
	}
	stack = zoomer->zoomStack();
	stack.replace(0, rect);
	zoomer->setZoomStack(stack, 0);
	zoomer->zoom(currentIndex);

	replot();
}

bool Mhisto::save_data()
{
	QString fileName = QFileDialog::getSaveFileName(this, tr("Save Data"),
					   "./",
					   tr("Text Data (*.txt)"));
	if (fileName.isEmpty()) {
		return true;
	}

	if (data_store.histosize == 0) {
		return false;
	}

	return data_store.save(fileName);
}

void Mhisto::set_accumulation(int a)
{
	switch (a) {
		case Qt::Unchecked:
			accumulation = false;
			break;
		case Qt::Checked:
			accumulation = true;
			break;
		default:
			;
	}
}

const HistoData &Mhisto::histo_data() const
{
	return data_store;
}

void Mhisto::erase_data_store()
{
	data_store.reset();
}
