/**
  @file
*/

#pragma once

#if defined _WIN32 || defined __CYGWIN__
	#ifdef SCTDC_EXPORTS
		#define SCTDCDLL_PUBLIC __declspec(dllexport)
		#define WIN32_LEAN_AND_MEAN
	#else
		#define SCTDCDLL_PUBLIC __declspec(dllimport)
	#endif
#elif __linux__
	#ifdef SCTDC_EXPORTS
		#define SCTDCDLL_PUBLIC __attribute__ ((visibility("default")))
	#else
		#define SCTDCDLL_PUBLIC
	#endif
  #include <sys/types.h>
#else
  #error platform is not supported
#endif

#include "scTDC_types.h"

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus


/**
  @brief    Gives an error description in a text form
  @param    err_code	Integer error code
  @param   err_msg		Pointer to put a text description.

  @note The err_msg array must be not shorted then ERRSTRLEN constant.

 */
SCTDCDLL_PUBLIC void sc_get_err_msg(int err_code, char *err_msg);

/**
 * @brief Check type of event data.
 * @param type Event type for comparison.
 * @param event Event under testing.
 * @param event_len_bytes Length of event in bytes.
 * @return int Result of comparison.
 *
 * Call this function to test whether event is special sign or not.
 *
 * Returns non zero value if event is of requested type otherwise returns zero.
 */
SCTDCDLL_PUBLIC int
sc_tdc_is_event
(const enum sc_event_type_index type,
const void *event,
const unsigned event_len_bytes);



// -------------------------- api vmaj2 -----------------------------------------

/**
 * @brief Initializes the hardware and loads the initial settings reading
 *  it from ini file.
 * @param ini_filename Name of the inifile used for initialization.
 * @return int device descriptor or error code if less then zero.
 *
 * The function must be called before any other operation and associates
 * an positive integer number (device descriptor) with the device. Device
 * descriptor must be used to identify hardware for any other operation.
 *
 * Hardware with the same serial number cannot be opened twice. Call
 * sc_tdc_deinit2() to close and deinitialize hardware.
 */
SCTDCDLL_PUBLIC int sc_tdc_init_inifile(const char *ini_filename);

/**
 * @brief Initializes the hardware and loads the initial settings taken
 * from array of sc_ConfigLine structures.
 * @param confline_array Array of sc_ConfigLine structures which contains
 * configuration needed.
 * @return int device descriptor or error code if less then zero.
 *
 * The function must be called before any other operation and associates an
 * positive integer number (device destriptor) with the device. Device
 * descriptor must be used to identify hardware for any other operation.
 *
 * Hardware with the same serial number cannot be opened twice. Call
 * sc_tdc_deinit2() to close and deinitialize hardware.
 *
 * Last structure of the array must contain sc_ConfigLine::section equal NULL.
 */
SCTDCDLL_PUBLIC int
sc_tdc_init_with_config_lines(const struct sc_ConfigLine *confline_array);

/**
 * @brief Deinitialize the hardware
 * @param dev_desc Device descriptor of the hardware to be deinitialized.
 * @return int 0 or error code.
 *
 * Call this function to release device and resources allocated to operate.
 * Hardware is deinitialized after the call.
 */
SCTDCDLL_PUBLIC int sc_tdc_deinit2(const int dev_desc);

/**
 * @brief Get data format of the tdc events.
 * @param dev_desc Device descriptor.
 * @param format Pointer on the structure where format should be placed.
 * @return int 0 or error code.
 *
 * Call this function to get event format got from tdc pipe.
 * See sc_tdc_pipe_open2() for details.
 */
SCTDCDLL_PUBLIC int
sc_tdc_get_format2(const int dev_desc, struct sc_tdc_format *format);

SCTDCDLL_PUBLIC int sc_tdc_set_complete_callback2
  (const int, void *owner, void (*cb)(void *, int));

/**
 * @brief Start measure.
 * @param dev_desc Device destriptor.
 * @param ms Exposure time in milliseconds.
 * @return int 0 or error code.
 *
 * Start measurement.
 */
SCTDCDLL_PUBLIC int sc_tdc_start_measure2(const int dev_desc, const int ms);

/**
 * @brief Stop measure.
 * @param dev_desc Device descriptor.
 * @return int 0 or error code.
 *
 * Stops started exposure.
 */
SCTDCDLL_PUBLIC int sc_tdc_interrupt2(const int dev_desc);

/**
 * @brief Get binsize in nanoseconds.
 * @param dev_desc Device descriptor.
 * @param binsize_ns Pointer where binsize should be stored.
 * @return int 0 or error code.
 */
SCTDCDLL_PUBLIC int sc_tdc_get_binsize2(const int dev_desc, double *binsize_ns);

/**
 * @brief Get running status.
 * @param dev_desc Device descriptor.
 * @param status Pointer where status info must be stored.
 * @return int 0 or error code.
 *
 * The function could be used to get status of hardware.
 * status 1 means idle state, 0 means that exposure is running.
 */
SCTDCDLL_PUBLIC int sc_tdc_get_status2(const int dev_desc, int *status);

SCTDCDLL_PUBLIC int sc_twi_write2
(const int,
const unsigned char address,
const unsigned char *data,
const size_t size,
const int stop);

SCTDCDLL_PUBLIC int sc_twi_read2
(const int,
const unsigned char address,
unsigned char *data,
const size_t size,
const int stop);

SCTDCDLL_PUBLIC int sc_twi_set_epot2
(const int, unsigned int epot, unsigned int value_number, unsigned char value);

/**
 * @brief Open data pipe.
 * @param dev_desc Device descriptor
 * @param type Pipe type.
 * @param params Pipe parameters.
 * @return Positive integer number which is pipe id, or error code.
  Please note that this is NOT buffer size.
 * @see enum sc_pipe_type_t
 * @see struct sc_pipe_dld_image_xy_params_t
 * @see struct sc_pipe_dld_image_xt_params_t
 * @see struct sc_pipe_dld_image_yt_params_t
 * @see struct sc_pipe_dld_image_3d_params_t
 * @see struct sc_pipe_dld_sum_histo_params_t
 * @see struct sc_pipe_tdc_histo_params_t
 * @see struct sc_pipe_statistics_params_t
 * @see struct sc_pipe_callback_params_t
 *
 * User can get as many pipes as required. It is not error to get two or more
 * pipes of the same type, but for example with different roi or binning
 * setting.
 * @note In case of internal memory allocation if data is not read from
 * the pipe after every exposure memory consumption of the application
 * will grow up. Please be sure that data is read.
 */
SCTDCDLL_PUBLIC int sc_pipe_open2
(const int dev_desc, const enum sc_pipe_type_t type, const void *params);

/**
 * @brief Close data pipe.
 * @param dev_desc Device descriptor.
 * @param pipe_id Pipe id.
 * @return 0 or error code.
 */
SCTDCDLL_PUBLIC int sc_pipe_close2(const int dev_desc, const int pipe_id);

/**
 * @brief Read data from pipe.
 * @param dev_desc Device descriptor.
 * @param pipe_id Pipe id.
 * @param buffer Pointe to pointer where data block must be stored.
 * @param timeout Timeout in millisecond.
 *
 * This function allocates memory with allocator callback from pipe
 * parameters structure if callback function was installed or allocates
 * memory internally. Then copies data from the last exposure and returns the
 * memory block in buffer. If memory was allocated internally it will be
 * deallocated when next call to the function will be performed or pipe will
 * be closed. If memory allocation callback function was installed in the
 * pipe parameters no any deallocation of the memory will be performed. User
 * must manage memory by him-/her- self.
 *
 * Function returns when: data is available or timeout or pipe is closed.
 */
SCTDCDLL_PUBLIC int sc_pipe_read2
(const int dev_desc, const int pipe_id, void **buffer, unsigned int timeout);

SCTDCDLL_PUBLIC int
sc_tdc_set_flim_scanner2
(const int dd,
unsigned short pixel_interval,
unsigned short pixel_count,
unsigned short line_count,
unsigned int line_delay_interval,
unsigned int multiline_count,
double *corr_table);

SCTDCDLL_PUBLIC int
sc_tdc_set_flim_scanner2ex
(const int dd,
unsigned short pixel_interval,
unsigned short pixel_count,
unsigned short line_count,
unsigned int line_delay_interval,
unsigned int multiline_count,
double *corr_table,
unsigned int flags);

SCTDCDLL_PUBLIC int
sc_tdc_is_event2
(const enum sc_event_type_index type,
const void *event,
const unsigned event_len_bytes);

SCTDCDLL_PUBLIC int sc_tdc_set_corrections2(const int, const int, const int *, const unsigned char *);
SCTDCDLL_PUBLIC int sc_tdc_set_common_shift2(const int, const int);
SCTDCDLL_PUBLIC int sc_tdc_set_modulo2(const int, const unsigned int);
SCTDCDLL_PUBLIC int sc_tdc_get_modulo2(const int, unsigned int *);
/**
 * @brief Open data pipe for tdc events.
 * @param dd Device descriptor.
 * @param internal_pipe_size Size of internal data buffer for events in bytes.
 * @param pipe_warning 90% pipe level callback function.
 * @param pipe_alert 99% pipe level callback function.
 * @return int 0 or error code.
 */
SCTDCDLL_PUBLIC int sc_tdc_pipe_open2
(const int dd, size_t internal_pipe_size,
const struct sc_PipeCbf *pipe_warning,
const struct sc_PipeCbf *pipe_alert);

/**
 * @brief Close data pipe for tdc events.
 * @param dd Device descriptor.
 * @return int 0 or error code.
 */
SCTDCDLL_PUBLIC int sc_tdc_pipe_close2(const int dd);

/**
 * @brief Read tdc events from the pipe.
 * @param dd Device descriptor.
 * @param buffer Space for events.
 * @param buffer_size_bytes Size of the buffer in bytes.
 * @param timeout Timeout in milliseconds.
 * @return ssize_t Amount of bytes were copied to the buffer or error code.
 * @see sc_tdc_get_format2(), sc_tdc_is_event2(), sc_tdc_is_event()
 */
SCTDCDLL_PUBLIC ssize_t sc_tdc_pipe_read2
(const int dd, void *buffer, size_t buffer_size_bytes, unsigned timeout);

/**
 * @brief Get configuration data.
 * @param dd Device descriptor.
 * @param key Configuration key in form 'section:key'
 * @param buf Space for configuration value as a string.
 * @param buf_size Size of buf in bytes.
 * @param def Default value.
 */
SCTDCDLL_PUBLIC int sc_tdc_config_get_key
(const int dd, const char *key, char *buf, size_t buf_size, const char *def);

/**
 * @brief Get scTDC version.
 */
SCTDCDLL_PUBLIC void sc_tdc_config_get_library_version(unsigned [3]);

//v1.1405

/**
 * @brief Get Device Properties.
 * @param dd Device descriptor.
 * @param params_num Number of properties structure.
 * @param params Properties structure casted to void pointer.
 *
 * params_num 1 coresponds to sc_DeviceProperties1 structure.
 * params_num 2 coresponds to sc_DeviceProperties2 structure.
 */
SCTDCDLL_PUBLIC int sc_tdc_get_device_properties
(const int dd, int params_num, void *params);

//v1.1406.1
SCTDCDLL_PUBLIC int sc_dld_set_hardware_binning
(const int dd, const struct sc3du_t *binning);

//v1.15xx

/**
  @brief Set Iteration Number.
  @param dd Device descriptor.
  @param itnum Number of iteration per measure.
  @return int 0 or error code.
*/
SCTDCDLL_PUBLIC int sc_tdc_set_iteration_number2(const int dd, int itnum);

//v1.1711.1

/**
  @brief Set cmos and smoother parameters.
  @param dd Device descriptor.
  @param params Cmos and smoother parameters structure.
  @return int 0 or error code.
*/

SCTDCDLL_PUBLIC int sc_tdc_set_cmos_and_smoothers_params
(const int dd, const struct sc_CmosSmootherParameters *params);

/**
  @brief Set blob parameters.
  @param dd Device descriptor.
  @param params Blob parameters.
  @return int 0 or error code.
*/

SCTDCDLL_PUBLIC int sc_tdc_set_blob_parameters
(const int dd, const struct sc_BlobParameters *params);

/**
  @brief Get cmos and smoother parameters.
  @param dd Device descriptor.
  @param params Cmos and smoother parameters structure.
  @return int 0 or error code.

  Function reads default configuration for cmos and smoother
  from the inifile. Setting parameters with
  sc_tdc_set_cmos_and_smoothers_params()
  does not change default parameters.
*/
SCTDCDLL_PUBLIC int sc_tdc_get_cmos_and_smoothers_params
(const int dd, struct sc_CmosSmootherParameters *params);

/**
  @brief Get blob parameters.
  @param dd Device descriptor.
  @param params Blob parameters structure.
  @return int 0 or error code.

  Function read default configuration for blob algorithm
  from inifile. Setting parameters with
  sc_tdc_set_blob_parameters does not change
  default parameters.
*/
SCTDCDLL_PUBLIC int sc_tdc_get_blob_parameters
(const int dd, struct sc_BlobParameters *params);

//v1.3001.0

/**
  @brief Set blob algorithm.
  @param dd Device descriptor.
  @param blob Library name where blob algorithm implemented.
  @return int 0 or error code.
*/
SCTDCDLL_PUBLIC int sc_tdc_set_blob
(const int dd, const char *blob);

/**
  @brief Get default blob algorithm.
  @param dd Device descriptor.
  @param str Buffer for blob name string.
  @param str_len Buffer for blob name string length.
  @param str_len_ret Used memory space for blob name string.
  @return int 0 or error code.
*/
SCTDCDLL_PUBLIC int sc_tdc_get_blob
(const int dd, char *str, size_t str_len, size_t *str_len_ret);

SCTDCDLL_PUBLIC int sc_tdc_zero_master_reset_counter
(const int dd);


#ifdef __cplusplus
}
#endif // __cplusplus

#define ERRSTRLEN				256
